<!--
SPDX-FileCopyrightText: 2019-2023 Intel Corporation

SPDX-License-Identifier: MIT
-->

{% extends "!layout.html" %}

          {% block sidebartitle %}

          {% if theme_display_version %}
            {%- set nav_version = version %}
            {% if READTHEDOCS and current_version %}
              {%- set nav_version = current_version %}
            {% endif %}
            {% if nav_version %}
              <div class="version">
                {{ nav_version }}
              </div>
            {% endif %}
          {% endif %}

          {% include "searchbox.html" %}

          {% endblock %}
